from typing import Dict

import holidays
from database_mysql_local.generic_crud import GenericCRUD
from logger_local.MetaLogger import MetaLogger

# from location_local.locations_local_crud import LocationsLocal
from .constants_event_local import EVENTS_LOCAL_CODE_LOGGER_OBJECT


# TODO: test
class HolidayLocal(
    # TODO Shall we change it to GenericCrudMl?
    GenericCRUD, metaclass=MetaLogger, object=EVENTS_LOCAL_CODE_LOGGER_OBJECT
):
    """HolidayLocal"""

    def __init__(self, is_test_data: bool = False) -> None:
        super().__init__(
            default_schema_name="event",
            default_table_name="event_table",
            default_view_table_name="event_view",
            default_column_name="event_id",
            is_test_data=is_test_data,
        )

    def insert_holiday(
        self, *, date: str, holiday: str, country: str, state: str = None
    ) -> int:
        """Inserts a new holiday and returns the new holiday_id"""

        # location_obj = LocationsLocal()
        # lang_code = "en"
        # all_details = {"coordinate": {"latitude": None, "longitude": None}, "address_local_language": None,
        #                "address_english": None, "neighborhood": None, "county": None, "region": None, "state": state,
        #                "country": country, "postal_code": None, "plus_code": None, "is_approved": False}
        # location_id = location_obj.get_location_ids(all_details, lang_code, True)
        location_id = 3  # TODO
        if self.holiday_exists(
            date=date, holiday=holiday, country=country, state=state
        ):
            event_table_data = {
                "start_timestamp": date,
                "location_id": location_id,
                "website_url": "test for holiday",
            }
            holiday_id = super().insert(data_dict=event_table_data)
            return holiday_id

    def update_holiday(self, *, holiday_id: int, holiday_data: Dict[str, any]) -> None:
        """Updates an existing holiday"""
        event_table_data = {
            "date": holiday_data.get("date"),
            "holiday": holiday_data.get("holiday"),
            "country": holiday_data.get("country"),
            "state": holiday_data.get("state"),
        }
        super().update_by_column_and_value(
            column_value=holiday_id, data_dict=event_table_data
        )

    def get_holiday_by_id(self, holiday_id: int) -> Dict[str, any]:
        """Gets holiday information by holiday_id"""
        holiday_dict = self.select_one_dict_by_column_and_value(column_value=holiday_id)
        return holiday_dict

    def delete_holiday_by_id(self, holiday_id: int) -> None:
        """Deletes a holiday by holiday_id"""
        self.delete_by_column_and_value(column_value=holiday_id)

    def holiday_exists(
        self, *, date: str, holiday: str, country: str, state: str = None
    ) -> bool:
        """Checks if a holiday with the given details already exists in the database"""
        # TODO: fix
        # conditions = {
        #     "date": date,
        #     "holiday": holiday,
        #     "country": country,
        #     "state": state
        # }
        existing_holiday = self.select_one_dict_by_column_and_value(
            view_table_name="event_view", column_name="event_id", column_value="2"
        )
        return existing_holiday is not None

    @staticmethod
    def fetch_holidays(country: str, year: int) -> list:
        """fetch us holidays"""
        holidays_list = sorted(holidays.country_holidays(country, years=year).keys())
        return holidays_list



from database_mysql_local.generic_crud import GenericCRUD
from database_mysql_local.table_columns import table_columns
from python_sdk_remote.utilities import validate_url

event_table_columns = table_columns["event_table"]


# Note: if you shange those messages, update the tests accordingly
class ValidateInputs(GenericCRUD):
    def __init__(self, is_test_data: bool = False) -> None:
        super().__init__(
            default_schema_name="event",
            default_column_name="event_id",
            default_table_name="event_table",
            default_view_table_name="event_view",
            is_test_data=is_test_data,
        )

    def validate_location_id(self, location_id: int) -> None:
        if location_id is None or not isinstance(location_id, int) or location_id < 0:
            raise ValueError(
                f"Invalid location_id: {location_id} (type: {type(location_id)})"
            )
        _location_id = self.select_one_value_by_column_and_value(
            schema_name="location",
            view_table_name="location_view",
            select_clause_value="location_id",
            column_name="location_id",
            column_value=location_id,
        )
        if _location_id is None:
            raise ValueError(
                f"Invalid location_id: {location_id} (not found in location.location_view)"
            )

    def validate_organizers_profile_id(self, organizers_profile_id: int) -> None:
        if (
            not organizers_profile_id
            or not isinstance(organizers_profile_id, int)
            or organizers_profile_id <= 0
        ):
            raise ValueError(
                f"Invalid organizers_profile_id: {organizers_profile_id} (type: {type(organizers_profile_id)})"
            )
        _organizers_profile_id = self.select_one_value_by_column_and_value(
            schema_name="profile",
            view_table_name="profile_view",
            select_clause_value="profile_id",
            column_name="profile_id",
            column_value=organizers_profile_id,
        )
        if _organizers_profile_id is None:
            raise ValueError(
                f"Invalid organizers_profile_id: {organizers_profile_id} (not found in profile.profile_view)"
            )

    def validate_input_dict(self, input_dict: dict) -> None:
        location_id = input_dict.get("location_id")
        organizers_profile_id = input_dict.get("organizers_profile_id")
        website_url = input_dict.get("website_url")
        facebook_event_url = input_dict.get("facebook_event_url")
        meetup_event_url = input_dict.get("meetup_event_url")
        registration_url = input_dict.get("registration_url")
        # TODO: allowed_keys = [...] and set(input_dict.keys()) == set(allowed_keys)
        self.validate_location_id(location_id)
        self.validate_organizers_profile_id(organizers_profile_id)

        if website_url and not validate_url(website_url):
            raise ValueError(f"Invalid input: website_url - {website_url}")
        if facebook_event_url and not validate_url(facebook_event_url):
            raise ValueError(
                f"Invalid input: facebook_event_url - {facebook_event_url}"
            )
        if meetup_event_url and not validate_url(meetup_event_url):
            raise ValueError(f"Invalid input: meetup_event_url - {meetup_event_url}")
        if registration_url and not validate_url(registration_url):
            raise ValueError(f"Invalid input: registration_url - {registration_url}")

        for key in list(input_dict):
            if key not in event_table_columns:
                input_dict.pop(key)

    # TODO Move this function to database-mysql-python-package and make it generic for all tables using Entity enum
    def validate_id(self, entity_id: int, schema: str = "event") -> None:
        try:
            entity_id = int(entity_id)
            if entity_id < 0:
                raise ValueError
        except ValueError:
            raise ValueError(f"Invalid {schema}_id: {entity_id}")
        assert ";" not in schema  # Prevent SQL Injection
        # value = self.select_one_value_by_column_and_value(
        value = self.select_one_value_by_where(
            schema_name=schema,
            view_table_name=schema + "_view",
            select_clause_value=schema + "_id",
            where="event_id = %s",
            params=(entity_id,),
        )
        if value is None:
            raise ValueError(
                f"Invalid {schema}_id: {entity_id} (not found in {schema}.{schema}_view)"
            )



import json
from http import HTTPStatus

from python_sdk_remote.http_response import create_authorization_http_headers
from user_context_remote.user_context import UserContext

# Import the function to be tested
from src.handler import createHandler, getEventByEventIdHandler

user_jwt = UserContext().get_user_jwt()


class TestCreate:
    @staticmethod
    def test_create_valid_input():
        event = {
            "body": json.dumps(
                {
                    "location_id": 0,  # TODO: get_test_location_id
                    "organizers_profile_id": 2,
                    "website_url": "https://example.com",
                    "facebook_event_url": "https://facebook.com/event",
                    "meetup_event_url": "https://meetup.com/event",
                    "registration_url": "https://example.com/register",
                    "isTestData": True,
                    "end_timestamp": None,
                }
            ),
            "headers": create_authorization_http_headers(user_jwt),
        }

        # Call the function
        response = createHandler(event, None)

        # Assert that the response is correct
        assert response["statusCode"] == HTTPStatus.OK.value, response

        read_event = {
            "pathParameters": {"eventId": json.loads(response["body"])["event_id"]},
            "body": json.dumps({"isTestData": True}),
            "headers": create_authorization_http_headers(user_jwt),
        }
        read_response = getEventByEventIdHandler(read_event, None)
        assert read_response["statusCode"] == HTTPStatus.OK.value, read_response

    @staticmethod
    def test_create_invalid_input():
        # Define a mock event with invalid input data
        invalid_event = {
            "body": json.dumps(
                {
                    "location_id": -1,
                    "organizers_profile_id": 2,
                    "website_url": "invalid url",
                    "facebook_event_url": "https://facebook.com/event",
                    "meetup_event_url": "https://meetup.com/event",
                    "registration_url": "https://example.com/register",
                }
            ),
            "headers": create_authorization_http_headers(user_jwt),
        }
        event = invalid_event

        # Call the function
        response = createHandler(event, None)

        # Assert that the response is correct
        assert response["statusCode"] != HTTPStatus.OK.value, response
        assert (
            "Invalid location_id: -1 (type: <class 'int'>)" in response["body"]
        ), response

        # Assert that the cursor did not execute any SQL queries
        # self.mock_cursor.execute.assert_not_called()


if __name__ == "__main__":
    test = TestCreate()

    test.test_create_valid_input()



import unittest
import json
from http import HTTPStatus
from python_sdk_remote.http_response import create_authorization_http_headers
from user_context_remote.user_context import UserContext
# from language_remote import lang_code

from src.handler import getEventByEventIdHandler, EventsLocal


user_jwt = UserContext().get_user_jwt()


class TestRead(unittest.TestCase):
    def test_read_valid_input(self):
        # Set up the mock event and connection
        event = {  # TODO Please make sure it will work in any database/environment
            "pathParameters": {
                "eventId": EventsLocal().get_test_event_id(),
            },
            "headers": create_authorization_http_headers(user_jwt),
        }

        # Call the function
        response = getEventByEventIdHandler(event, None)

        # Assert that the response is correct
        assert response["statusCode"] == HTTPStatus.OK.value, response
        select_clause = "event_id, location_id, organizers_profile_id, website_url, facebook_event_url, meetup_event_url, registration_url"
        assert all(
            key in json.loads(response["body"]) for key in select_clause.split(", ")
        ), response

    def test_read_invalid_event_id(self):
        # Define a mock event with an invalid event ID
        invalid_event = {
            "pathParameters": {"eventId": "invalid"},
            "headers": create_authorization_http_headers(user_jwt),
        }

        # Call the function
        response = getEventByEventIdHandler(invalid_event, None)

        # Assert that the response is correct
        # TODO change Magic number to enum/const
        assert response["statusCode"] != HTTPStatus.OK.value
        assert "Invalid event_id: invalid" in response["body"]

    def test_read_event_not_found(self):
        # Set up the mock event and connection
        event = {
            "pathParameters": {"eventId": "0"},
            "headers": create_authorization_http_headers(user_jwt),
        }

        # Call the function
        response = getEventByEventIdHandler(event, None)

        # Assert that the response is correct
        assert response["statusCode"] != HTTPStatus.OK.value, response
        assert "error" in response["body"], response

    @staticmethod
    def test_get_number_of_events():
        events_local = EventsLocal()
        current_number_of_events = events_local.get_number_of_events()
        sql_statement = "SELECT COUNT(*) FROM event.event_view"
        events_local.cursor.execute(sql_statement=sql_statement)
        number_of_events_from_sql = events_local.cursor.fetchone()[0]
        assert current_number_of_events == number_of_events_from_sql


# Call
if __name__ == "__main__":
    unittest.main()

# test_read1=test_read()
# test_read1.setUp()
# test_read1.test_read_valid_input()



# TODO Can we use event-local package instead of this file
from python_sdk_remote.our_object import OurObject

ENTITY_NAME = "Event"


class Event(OurObject):
    # def __init__(self, **kwargs):
    def __init__(self, entity_name=ENTITY_NAME, **kwargs):
        super().__init__(entity_name, **kwargs)

        event_fields = {
            "event_id",
            "number",
            "identifier",
            "name",
            "description",
            "type",
            "sub_type",
            "genre",
            "sub_genre",
            "segement",
            "visibility_id",
            "start_timestamp",
            "end_timestamp",
            "access_start_timestamp",
            "created_timestamp",
            "updated_timestamp",
            "location_id",
            "location_id_temp",
            "organizers_profile_id",
            "parent_event_id",
            "facebook_event_url",
            "meetup_event_url",
            "registration_url",
            "website_url",
            "is_beakfast",
            "is_lunch",
            "is_dinner",
            "is_family",
            "is_event_approved",
            "is_event_has_promoters",
            "is_paid",
            "is_can_be_paid_at_the_entrance_in_cash",
            "is_require_end_user_registration",
            "is_require_end_user_arrival_confirmation",
            "is_require_organizer_registration_confirmation",
            "is_show_end_timestamp",
            "is_test_data",
            "is_waitinglist",
            "created_effective_profile_id",
            "created_effective_user_id",
            "created_real_user_id",
            "created_user_id",
            "updated_effective_profile_id",
            "updated_effective_user_id",
            "updated_real_user_id",
            "updated_user_id",
        }

        # for field, value in kwargs.items():
        #     if field in event_fields:
        #         setattr(self, field, value)

    # Mandatory pure virtual method from OurObject
    def get_name(self):
        print(f"{ENTITY_NAME} get_name() self.fields.display_as={self.fields.display_as}")
        return self.fields.display_as

    # def get_name(self):
    #     return self.name

    # def __str__(self):
    #     return "event: " + str(self.__dict__)

    # def __repr__(self):
    #     return "event: " + str(self.__dict__)

    # def to_dict(self):
    #     return self.__dict__
